proc start_step { step } {
  set stopFile ".stop.rst"
  if {[file isfile .stop.rst]} {
    puts ""
    puts "*** Halting run - EA reset detected ***"
    puts ""
    puts ""
    return -code error
  }
  set beginFile ".$step.begin.rst"
  set platform "$::tcl_platform(platform)"
  set user "$::tcl_platform(user)"
  set pid [pid]
  set host ""
  if { [string equal $platform unix] } {
    if { [info exist ::env(HOSTNAME)] } {
      set host $::env(HOSTNAME)
    }
  } else {
    if { [info exist ::env(COMPUTERNAME)] } {
      set host $::env(COMPUTERNAME)
    }
  }
  set ch [open $beginFile w]
  puts $ch "<?xml version=\"1.0\"?>"
  puts $ch "<ProcessHandle Version=\"1\" Minor=\"0\">"
  puts $ch "    <Process Command=\".planAhead.\" Owner=\"$user\" Host=\"$host\" Pid=\"$pid\">"
  puts $ch "    </Process>"
  puts $ch "</ProcessHandle>"
  close $ch
}

proc end_step { step } {
  set endFile ".$step.end.rst"
  set ch [open $endFile w]
  close $ch
}

proc step_failed { step } {
  set endFile ".$step.error.rst"
  set ch [open $endFile w]
  close $ch
}

set_msg_config -id {HDL 9-1061} -limit 100000
set_msg_config -id {HDL 9-1654} -limit 100000

start_step init_design
set rc [catch {
  create_msg_db init_design.pb
  set_param xicom.use_bs_reader 1
  debug::add_scope template.lib 1
  create_project -in_memory -part xc7a100tcsg324-1
  set_property board_part digilentinc.com:nexys4_ddr:part0:1.1 [current_project]
  set_property design_mode GateLvl [current_fileset]
  set_property webtalk.parent_dir C:/Users/tkappenm/Desktop/LooperDemo/LooperDemo.cache/wt [current_project]
  set_property parent.project_path C:/Users/tkappenm/Desktop/LooperDemo/LooperDemo.xpr [current_project]
  set_property ip_repo_paths c:/Users/tkappenm/Desktop/LooperDemo/LooperDemo.cache/ip [current_project]
  set_property ip_output_repo c:/Users/tkappenm/Desktop/LooperDemo/LooperDemo.cache/ip [current_project]
  add_files -quiet C:/Users/tkappenm/Desktop/LooperDemo/LooperDemo.runs/synth_1/looper1_1.dcp
  add_files -quiet c:/Users/tkappenm/Desktop/LooperDemo/LooperDemo.srcs/sources_1/ip/mig_7series_0/mig_7series_0.dcp
  set_property netlist_only true [get_files c:/Users/tkappenm/Desktop/LooperDemo/LooperDemo.srcs/sources_1/ip/mig_7series_0/mig_7series_0.dcp]
  add_files -quiet c:/Users/tkappenm/Desktop/LooperDemo/LooperDemo.srcs/sources_1/ip/clk_wiz_0/clk_wiz_0.dcp
  set_property netlist_only true [get_files c:/Users/tkappenm/Desktop/LooperDemo/LooperDemo.srcs/sources_1/ip/clk_wiz_0/clk_wiz_0.dcp]
  add_files -quiet C:/Users/tkappenm/Desktop/LooperDemo/LooperDemo.runs/xadc_wiz_0_synth_1/xadc_wiz_0.dcp
  set_property netlist_only true [get_files C:/Users/tkappenm/Desktop/LooperDemo/LooperDemo.runs/xadc_wiz_0_synth_1/xadc_wiz_0.dcp]
  read_xdc -ref mig_7series_0 c:/Users/tkappenm/Desktop/LooperDemo/LooperDemo.srcs/sources_1/ip/mig_7series_0/mig_7series_0/user_design/constraints/mig_7series_0.xdc
  set_property processing_order EARLY [get_files c:/Users/tkappenm/Desktop/LooperDemo/LooperDemo.srcs/sources_1/ip/mig_7series_0/mig_7series_0/user_design/constraints/mig_7series_0.xdc]
  read_xdc -prop_thru_buffers -ref clk_wiz_0 -cells inst c:/Users/tkappenm/Desktop/LooperDemo/LooperDemo.srcs/sources_1/ip/clk_wiz_0/clk_wiz_0_board.xdc
  set_property processing_order EARLY [get_files c:/Users/tkappenm/Desktop/LooperDemo/LooperDemo.srcs/sources_1/ip/clk_wiz_0/clk_wiz_0_board.xdc]
  read_xdc -ref clk_wiz_0 -cells inst c:/Users/tkappenm/Desktop/LooperDemo/LooperDemo.srcs/sources_1/ip/clk_wiz_0/clk_wiz_0.xdc
  set_property processing_order EARLY [get_files c:/Users/tkappenm/Desktop/LooperDemo/LooperDemo.srcs/sources_1/ip/clk_wiz_0/clk_wiz_0.xdc]
  read_xdc -mode out_of_context -ref xadc_wiz_0 -cells inst c:/Users/tkappenm/Desktop/LooperDemo/LooperDemo.srcs/sources_1/ip/xadc_wiz_0/xadc_wiz_0_ooc.xdc
  set_property processing_order EARLY [get_files c:/Users/tkappenm/Desktop/LooperDemo/LooperDemo.srcs/sources_1/ip/xadc_wiz_0/xadc_wiz_0_ooc.xdc]
  read_xdc -ref xadc_wiz_0 -cells inst c:/Users/tkappenm/Desktop/LooperDemo/LooperDemo.srcs/sources_1/ip/xadc_wiz_0/xadc_wiz_0.xdc
  set_property processing_order EARLY [get_files c:/Users/tkappenm/Desktop/LooperDemo/LooperDemo.srcs/sources_1/ip/xadc_wiz_0/xadc_wiz_0.xdc]
  read_xdc C:/Users/tkappenm/Desktop/LooperDemo/LooperDemo.srcs/constrs_1/imports/XDC/Nexys4DDR_Master.xdc
  link_design -top looper1_1 -part xc7a100tcsg324-1
  close_msg_db -file init_design.pb
} RESULT]
if {$rc} {
  step_failed init_design
  return -code error $RESULT
} else {
  end_step init_design
}

start_step opt_design
set rc [catch {
  create_msg_db opt_design.pb
  catch {write_debug_probes -quiet -force debug_nets}
  opt_design 
  write_checkpoint -force looper1_1_opt.dcp
  catch {report_drc -file looper1_1_drc_opted.rpt}
  close_msg_db -file opt_design.pb
} RESULT]
if {$rc} {
  step_failed opt_design
  return -code error $RESULT
} else {
  end_step opt_design
}

start_step place_design
set rc [catch {
  create_msg_db place_design.pb
  catch {write_hwdef -file looper1_1.hwdef}
  place_design 
  write_checkpoint -force looper1_1_placed.dcp
  catch { report_io -file looper1_1_io_placed.rpt }
  catch { report_utilization -file looper1_1_utilization_placed.rpt -pb looper1_1_utilization_placed.pb }
  catch { report_control_sets -verbose -file looper1_1_control_sets_placed.rpt }
  close_msg_db -file place_design.pb
} RESULT]
if {$rc} {
  step_failed place_design
  return -code error $RESULT
} else {
  end_step place_design
}

start_step route_design
set rc [catch {
  create_msg_db route_design.pb
  route_design 
  write_checkpoint -force looper1_1_routed.dcp
  catch { report_drc -file looper1_1_drc_routed.rpt -pb looper1_1_drc_routed.pb }
  catch { report_timing_summary -warn_on_violation -max_paths 10 -file looper1_1_timing_summary_routed.rpt -rpx looper1_1_timing_summary_routed.rpx }
  catch { report_power -file looper1_1_power_routed.rpt -pb looper1_1_power_summary_routed.pb }
  catch { report_route_status -file looper1_1_route_status.rpt -pb looper1_1_route_status.pb }
  catch { report_clock_utilization -file looper1_1_clock_utilization_routed.rpt }
  close_msg_db -file route_design.pb
} RESULT]
if {$rc} {
  step_failed route_design
  return -code error $RESULT
} else {
  end_step route_design
}

start_step write_bitstream
set rc [catch {
  create_msg_db write_bitstream.pb
  write_bitstream -force looper1_1.bit 
  catch { write_sysdef -hwdef looper1_1.hwdef -bitfile looper1_1.bit -meminfo looper1_1.mmi -ltxfile debug_nets.ltx -file looper1_1.sysdef }
  close_msg_db -file write_bitstream.pb
} RESULT]
if {$rc} {
  step_failed write_bitstream
  return -code error $RESULT
} else {
  end_step write_bitstream
}

